#ifndef __XPRT__
#define __XPRT__
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

/* -------------------------- DATA TYPES ------------------------- */

typedef double *Vector;
typedef double **Matrix;

typedef int CoordinateNumber;
typedef int ExampleNumber;

typedef Vector InputVector;
typedef Vector OutputVector;
typedef Vector WeightVector;
typedef Vector ExampleVector;

typedef Matrix WeightMatrix;

enum ModelType {gaussian=0, bernoulli=1};
typedef enum ModelType ModelType;

typedef struct {
   ExampleVector h;
   WeightVector wt;
   struct NodeStruct *node1, *node2;
} Gate;

typedef struct {
   WeightMatrix wt;
   ExampleVector var;
   double phi;
} Xprt;

typedef struct NodeStruct {
   Gate *gate;
   Xprt *xprt;
} Node;

/* ----------------------- GLOBAL VARIABLES ---------------------- */

#ifndef __GLOBALS__
#define __GLOBALS__

extern InputVector *gX;	
extern ExampleVector *gY;	
extern CoordinateNumber gDimX;
extern CoordinateNumber gDimY;
extern ExampleNumber gNumPts;
extern ModelType gXprtModel;

#endif

/* ---------------------- CONSTANTS + MACROS ---------------------- */

#define IsXprt(node) (node->xprt)
#define IsGate(node) (node->gate)

#define kStrLen		256
#define kNumArgs	2
#define kNumEpochs	25
#define kBernoulliPhi	1.0
#define kIrlsMaxLoops	20
#define kIrlsErrCrit	0.000001
#define kWtScale	0.5	

/* ------------------------- compute.c ----------------------------- */

void InitIRLS(ExampleVector *w, ExampleVector *e, InputVector *rhs, 
              Matrix *mx, int **indx); 
void IRLS(ExampleVector y, ExampleVector yWts, WeightVector beta, 
          ModelType model, double phi);
void LuDecomp(Matrix mx, int n, int *indx);
void LuBackSub(Matrix mx, int n, int *indx, Vector vect);

/* --------------------------- main.c ------------------------------ */

Node *SetUp(int argc, char **argv);
ExampleVector Compute(Node *node);
void Update(Node *node, ExampleVector hp);
void NetOut(Node *node, InputVector x, OutputVector y);
void NetTest(Node *tree);

/* ---------------------------- new.c ------------------------------- */

Node *NewNode(void);
Xprt *NewXprt(void);
Gate *NewGate(void);
void AddLayer(Node *node, int *nSplits);
Matrix NewMatrix(int nRows, int nColumns);
Vector NewVector(int n);

/* --------------------------- utils.c ------------------------------ */

void Error(char *text);
void *SafeCalloc(int n, unsigned size);
void *SafeMalloc(unsigned size);
FILE *SafeOpen(char *fname, char *op);
double Rand1(void);

/* --------------------------- print.c ------------------------------ */

void PrintVector(Vector v, int n);
void PrintMatrix(Matrix mx, int nRows, int nColumns);
void PrintTree(Node *tree);

/* ------------------------------------------------------------------ */

#endif

